#!/usr/bin/env python
###########################################################################
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# PyQt port in November 2010 by Hans-Peter Jansen <hpj@urpla.net>
#
###########################################################################

import sys
import random
# This is only needed for Python v2 but is harmless for Python v3.
import sip
sip.setapi('QString', 2)    # converts QStrings to unicode objects internally

from PyQt4 import QtCore, QtNetwork, QtGui


class Client(QtGui.QDialog):
    def __init__(self, parent = None):
        super(Client, self).__init__(parent)
        self.blockSize = 0
        self.currentFortune = None
        hostLabel = QtGui.QLabel(self.tr("&Server name:"))
        self.hostLineEdit = QtGui.QLineEdit("fortune")
        hostLabel.setBuddy(self.hostLineEdit)
        self.statusLabel = QtGui.QLabel(
            self.tr("This examples requires that you run the "
                    "Fortune Server example as well."))
        self.getFortuneButton = QtGui.QPushButton(self.tr("Get Fortune"))
        self.getFortuneButton.setDefault(True)
        quitButton = QtGui.QPushButton(self.tr("Quit"))
        buttonBox = QtGui.QDialogButtonBox()
        buttonBox.addButton(self.getFortuneButton,
                            QtGui.QDialogButtonBox.ActionRole)
        buttonBox.addButton(quitButton, QtGui.QDialogButtonBox.RejectRole)
        self.socket = QtNetwork.QLocalSocket()
        self.hostLineEdit.textChanged.connect(self.enableGetFortuneButton)
        self.getFortuneButton.clicked.connect(self.requestNewFortune)
        quitButton.clicked.connect(self.close)
        self.socket.readyRead.connect(self.readFortune)
        self.socket.error.connect(self.displayError)
        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(hostLabel, 0, 0)
        mainLayout.addWidget(self.hostLineEdit, 0, 1)
        mainLayout.addWidget(self.statusLabel, 2, 0, 1, 2)
        mainLayout.addWidget(buttonBox, 3, 0, 1, 2)
        self.setLayout(mainLayout)
        self.setWindowTitle(self.tr("Fortune Client"))
        self.hostLineEdit.setFocus()

    def requestNewFortune(self):
        self.getFortuneButton.setEnabled(False)
        self.blockSize = 0
        self.socket.abort()
        self.socket.connectToServer(self.hostLineEdit.text())

    def readFortune(self):
        ins = QtCore.QDataStream(self.socket)
        ins.setVersion(QtCore.QDataStream.Qt_4_0)
        if self.blockSize == 0:
            if self.socket.bytesAvailable() < 2:
                return
            self.blockSize = ins.readUInt16()
        if ins.atEnd():
            return
        nextFortune = ins.readQString()
        if nextFortune == self.currentFortune:
            QtCore.QTimer.singleShot(0, self.requestNewFortune)
            return
 
        self.currentFortune = nextFortune
        self.statusLabel.setText(self.currentFortune)
        self.getFortuneButton.setEnabled(True)

    def displayError(self, socketError):
        errors = {
            QtNetwork.QLocalSocket.ServerNotFoundError:
                 self.tr("The host was not found. Please check the "
                         "host name and port settings."),

            QtNetwork.QLocalSocket.ConnectionRefusedError:
                 self.tr("The connection was refused by the peer. "
                         "Make sure the fortune server is running, "
                         "and check that the host name and port "
                         "settings are correct."),

            QtNetwork.QLocalSocket.PeerClosedError:
                None,
        }
        msg = errors.get(socketError,
                         self.tr("The following error occurred: %s." %
                                 self.socket.errorString()))
        if msg is not None:
            QtGui.QMessageBox.information(self,
                                          self.tr("Fortune Client"),
                                          msg)

        self.getFortuneButton.setEnabled(True)

    def enableGetFortuneButton(self):
       self.getFortuneButton.setEnabled(self.hostLineEdit.text() != "")

    def tr(self, s):
        return QtGui.QApplication.translate("Client", s, None,
                                            QtGui.QApplication.UnicodeUTF8)


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    client = Client()
    client.show()
    sys.exit(app.exec_())

