#!/usr/bin/env python
###########################################################################
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# PyQt port in November 2010 by Hans-Peter Jansen <hpj@urpla.net>
#
###########################################################################

import sys
import random
# This is only needed for Python v2 but is harmless for Python v3.
import sip
sip.setapi('QString', 2)    # converts QStrings to unicode objects internally

from PyQt4 import QtCore, QtNetwork, QtGui


class Server(QtGui.QDialog):
    def __init__(self, parent = None):
        super(Server, self).__init__(parent)
        statusLabel = QtGui.QLabel()
        quitButton = QtGui.QPushButton(self.tr("Quit"))
        quitButton.setAutoDefault(False)
        self.fortunes = (
            self.tr("You've been leading a dog's life. Stay off the furniture."),
            self.tr("You've got to think about tomorrow."),
            self.tr("You will be surprised by a loud noise."),
            self.tr("You will feel hungry again in another hour."),
            self.tr("You might have mail."),
            self.tr("You cannot kill time without injuring eternity."),
            self.tr("Computers are not intelligent. They only think they are."),
        )

        self.server = QtNetwork.QLocalServer()
        if not self.server.listen("fortune"):
            QtGui.QMessageBox.critical(self,
                                       self.tr("Fortune Server"),
                                       self.tr("Unable to start the server: %s." %
                                                self.server.errorString()))
            self.close()
            return

        statusLabel.setText(self.tr("The server is running.\n"
                                    "Run the Fortune Client example now."))
        quitButton.clicked.connect(self.close)
        self.server.newConnection.connect(self.sendFortune)
        buttonLayout = QtGui.QHBoxLayout()
        buttonLayout.addStretch(1)
        buttonLayout.addWidget(quitButton)
        buttonLayout.addStretch(1)
        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(statusLabel)
        mainLayout.addLayout(buttonLayout)
        self.setLayout(mainLayout)
        self.setWindowTitle(self.tr("Fortune Server"))


    def sendFortune(self):
        block = QtCore.QByteArray()
        out = QtCore.QDataStream(block, QtCore.QIODevice.WriteOnly)
        out.setVersion(QtCore.QDataStream.Qt_4_0)
        out.writeUInt16(0)
        out.writeQString(random.choice(self.fortunes))
        out.device().seek(0)
        out.writeUInt16(block.size() - 2)
        clientConnection = self.server.nextPendingConnection()
        clientConnection.disconnected.connect(clientConnection.deleteLater)
        clientConnection.write(block)
        clientConnection.flush()
        clientConnection.disconnectFromServer()

    def tr(self, s):
        return QtGui.QApplication.translate("Server", s, None,
                                            QtGui.QApplication.UnicodeUTF8)


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    server = Server()
    server.show()
    sys.exit(app.exec_())

